#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

""" gitlab group tools for pulling all repositories"""

from urllib.request import urlopen
import subprocess
import json
import os
import sys
import argparse


class GLPull(object):

    """ GitLab Clone all group repositories"""

    def __init__(self, options):
        """ the main program function

        :param options: parser options
        :type options: :class:`argparse.Namespace`
        """
        self.__grpurl = options.grpurl
        gfilters = options.args or ["tango-ds"]
        self.__filters = [gf.lower() for gf in gfilters]

    def run(self):
        """ the main program function
        """
        # fetch all subgroups
        groups = json.loads(urlopen(self.__grpurl).read().decode())
        for sg in groups:
            found = False
            for flt in self.__filters:
                if sg["full_path"].lower().startswith(flt):
                    found = True
            if found:
                filepath = sg["full_name"].replace(" / ", "/")
                urlpath = sg["full_name"].replace(" / ", "%2F")
                if os.path.exists(filepath):
                    print("checking %s" % filepath)
                    sgurl = "%s/%s/projects" % (self.__grpurl, urlpath)
                    # fetch all projects of the current subgroup
                    projects = json.loads(urlopen(sgurl).read().decode())
                    for pr in projects:
                        purl = pr["http_url_to_repo"]
                        if os.path.exists("%s/%s" % (filepath, pr["name"])):
                            try:
                                pullcmd = "cd %s/%s; git pull" % (
                                    filepath, pr["name"])
                                print(pullcmd)
                                process = subprocess.Popen(
                                    pullcmd,
                                    stdout=subprocess.PIPE,
                                    shell=True)
                                proc_stdout = process.communicate()[0].strip()
                                print(proc_stdout)
                            except Exception as e:
                                print("Error on %s: %s" % (purl, str(e)))


def main():
    """ the main program function
    """

    #: pipe arguments
    pipe = ""
    if not sys.stdin.isatty():
        pp = sys.stdin.readlines()
        #: system pipe
        pipe = "".join(pp)

    description = "Command-line tool for pulling all repositories " \
                  "of gitlab (sub-)group"

    epilog = 'examples:\n' \
        '  glpull  -a \n\n' \
        '    - pull all repositories of "tango-ds" group\n\n' \
        '  glpull tango-ds/DeviceClasses \n\n' \
        '    - pull all repositories of "tango-ds/DeviceClasses" ' \
        'subgroup\n\n'

    parser = argparse.ArgumentParser(
        description=description, epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument(
        'args', metavar='filter', type=str, nargs='?',
        help='group filter, e.g. "tango-ds"')
    parser.add_argument(
        "-g", "--groupurl",
        help='default: "https://eosc-pan-git.desy.de/api/v4/groups"',
        dest="grpurl", default="https://eosc-pan-git.desy.de/api/v4/groups")
    parser.add_argument(
        "-a", "--all", action="store_true",
        default=False, dest="all",
        help="set filter to 'tango-ds'")

    try:
        options = parser.parse_args()
    except Exception as e:
        sys.stderr.write("Error: %s\n" % str(e))
        sys.stderr.flush()
        parser.print_help()
        print("")
        sys.exit(255)

    #: command-line and pipe arguments
    parg = []
    if hasattr(options, "args"):
        parg = [options.args] if options.args else []
    if pipe:
        parg.append(pipe)
    if options.all:
        parg.append("tango-ds")
    options.args = parg

    if not options.args:
        parser.print_help()
        print("")
        sys.exit(255)

    try:
        command = GLPull(options)
        command.run()
    except Exception as e:
        sys.stderr.write("Error: glpull interrupted with:")
        sys.stderr.write(str(e))
        sys.stderr.flush()
        sys.exit(255)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

""" gitlab tools for uploading a local git repository to a remote gitlab"""

import subprocess
import sys
import argparse


class GLUpload(object):

    """ GitLab upload a local git repository to a remote gitlab"""

    def __init__(self, options):
        """ the main program function

        :param options: parser options
        :type options: :class:`argparse.Namespace`
        """
        self.__user = options.user
        self.__giturl = options.giturl or \
            "https://eosc-pan-git.desy.de/tango-ds/"
        self.__gitdir = options.args[0] if len(options.args) > 0 else ""
        self.__url = "%s/%s.git" % (self.__giturl, self.__gitdir) \
                     if self.__gitdir else options.url
        self.__localdir = options.localdir or "."

    def run(self):
        """ the main program function
        """

        addcmd = 'cd %s; git remote add gitlab %s' \
            % (self.__localdir, self.__url)
        pushcmd = 'cd %s; git push -u gitlab --all ; ' \
                  'git push -u gitlab --tags' \
                  % self.__localdir
        if self.__user:
            addcmd = addcmd.replace("://", "://%s@" % self.__user)
        commands = [addcmd, pushcmd]
        if self.__url:
            for cmd in commands:
                try:
                    print(cmd)
                    process = subprocess.Popen(
                        cmd,
                        stdout=subprocess.PIPE,
                        shell=True)
                    proc_stdout = process.communicate()[0].strip()
                    if hasattr(proc_stdout, "decode"):
                        print(proc_stdout.decode())
                except Exception as e:
                    print("Error: %s" % (str(e)))


def main():
    """ the main program function
    """

    #: pipe arguments
    pipe = ""
    if not sys.stdin.isatty():
        pp = sys.stdin.readlines()
        #: system pipe
        pipe = "".join(pp)

    description = "Command-line tool to upload a local git repository" \
                  " to a remote git repository"

    epilog = 'examples:\n' \
        '  glupload DeviceClasses/CounterTimer/DGG2 \n\n' \
        '    - upload a local git repository in the "." directory ' \
        'to https://eosc-pan-git.desy.de/tango-ds/DeviceClasses/' \
        'CounterTimer/DGG2.git \n\n' \

    parser = argparse.ArgumentParser(
        description=description, epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument(
        'args', metavar='group', type=str, nargs='?',
        help='gitlab group, '
        'e.g. "DeviceClasses/CounterTimer/DGG2"')
    parser.add_argument(
        "-g", "--git-url",
        help='root git url, '
        'default: "https://eosc-pan-git.desy.de/tango-ds/"',
        dest="giturl",
        default="https://eosc-pan-git.desy.de/tango-ds/")
    parser.add_argument(
        "-f", "--url",
        help='full git url with git subgroups and a project name, '
        'e.g.: "https://eosc-pan-git.desy.de/tango-ds/'
        'deviceclasses/countertimer/gdd2"',
        dest="url",
        default="")
    parser.add_argument(
        "-u", "--user",
        help="gitlab user",
        dest="user", default="")
    parser.add_argument(
        "-l", "--local-dir",
        help='local git directory, e.g.: "DGG2"',
        dest="localdir",
        default=".")

    try:
        options = parser.parse_args()
    except Exception as e:
        sys.stderr.write("Error: %s\n" % str(e))
        sys.stderr.flush()
        parser.print_help()
        print("")
        sys.exit(255)

    #: command-line and pipe arguments
    parg = []
    if hasattr(options, "args"):
        parg = [options.args] if options.args else []
    if pipe:
        parg.append(pipe)
    options.args = parg

    if not options.args and not options.url:
        parser.print_help()
        print("")
        sys.exit(255)

    try:
        command = GLUpload(options)
        command.run()
    except Exception as e:
        sys.stderr.write("Error: glupload interrupted with:")
        sys.stderr.write(str(e))
        sys.stderr.flush()
        sys.exit(255)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

""" gitlab group tools for cloning all repositories"""

from urllib.request import urlopen
import subprocess
import json
import os
import sys
import shlex
import getpass
import argparse


class GLClone(object):

    """ GitLab Clone all group repositories"""

    def __init__(self, options):
        """ the main program function

        :param options: parser options
        :type options: :class:`argparse.Namespace`
        """
        self.__user = options.user
        self.__grpurl = options.grpurl
        gfilters = options.args or ["tango-ds"]
        self.__filters = [gf.lower() for gf in gfilters]

    def run(self):
        """ the main program function
        """
        processes = []
        # fetch all subgroups
        groups = json.loads(urlopen(self.__grpurl).read().decode())
        for sg in groups:
            found = False
            for flt in self.__filters:
                if sg["full_path"].lower().startswith(flt):
                    found = True
            if found:
                filepath = sg["full_name"].replace(" / ", "/")
                urlpath = sg["full_name"].replace(" / ", "%2F")
                if not os.path.exists(filepath):
                    os.makedirs(filepath)
                print("checking %s" % filepath)
                sgurl = "%s/%s/projects" % (self.__grpurl, urlpath)
                # fetch all projects of the current subgroup
                projects = json.loads(urlopen(sgurl).read().decode())
                for pr in projects:
                    purl = pr["http_url_to_repo"]
                    if not os.path.exists("%s/%s" % (filepath, pr["name"])):
                        clonecmd = 'git clone %s %s/%s' % (
                            purl, filepath, pr["name"])
                        if self.__user:
                            clonecmd = clonecmd.replace(
                                "://", "://%s@" % self.__user)
                        print(clonecmd)
                        try:
                            command = shlex.split(clonecmd)
                            p = subprocess.Popen(command)
                            processes.append(p)
                        except Exception as e:
                            print("Error on %s: %s" % (purl, str(e)))

        print("Waiting for subprocesses")
        [p.wait() for p in processes]


def main():
    """ the main program function
    """

    #: pipe arguments
    pipe = ""
    if not sys.stdin.isatty():
        pp = sys.stdin.readlines()
        #: system pipe
        pipe = "".join(pp)

    description = "Command-line tool for cloning all repositories " \
                  "of gitlab (sub-)group"

    epilog = 'examples:\n' \
        '  glclone  -a \n\n' \
        '    - clone all repositories of "tango-ds" group\n\n' \
        '  glclone  -a -l  \n\n' \
        '    - clone all repositories of tango-ds group' \
        ' with gitlab user defined by local user\n\n' \
        '  glclone tango-ds/DeviceClasses \n\n' \
        '    - clone all repositories of "tango-ds/DeviceClasses" ' \
        'subgroup\n\n'

    parser = argparse.ArgumentParser(
        description=description, epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument(
        'args', metavar='filter', type=str, nargs='?',
        help='group filter, e.g. "tango-ds"')
    parser.add_argument(
        "-u", "--user",
        help="gitlab user",
        dest="user", default="")
    parser.add_argument(
        "-g", "--groupurl",
        help='group url, '
        'default: "https://eosc-pan-git.desy.de/api/v4/groups"',
        dest="grpurl", default="https://eosc-pan-git.desy.de/api/v4/groups")
    parser.add_argument(
        "-l", "--local-user", action="store_true",
        default=False, dest="local",
        help="get local user")
    parser.add_argument(
        "-a", "--all", action="store_true",
        default=False, dest="all",
        help="set filter to 'tango-ds'")

    try:
        options = parser.parse_args()
    except Exception as e:
        sys.stderr.write("Error: %s\n" % str(e))
        sys.stderr.flush()
        parser.print_help()
        print("")
        sys.exit(255)

    #: command-line and pipe arguments
    parg = []
    if hasattr(options, "args"):
        parg = [options.args] if options.args else []
    if pipe:
        parg.append(pipe)
    if options.all:
        parg.append("tango-ds")
    options.args = parg

    if not options.args:
        parser.print_help()
        print("")
        sys.exit(255)

    if not options.user and options.local:
        options.user = getpass.getuser()
    try:
        command = GLClone(options)
        command.run()
    except Exception as e:
        sys.stderr.write("Error: glclone interrupted with:")
        sys.stderr.write(str(e))
        sys.stderr.flush()
        sys.exit(255)


if __name__ == "__main__":
    main()



#!/usr/bin/env python
#   This file is part of nexdatas - Tango Server for NeXus data writer
#
#    Copyright (C) 2012-2018 DESY, Jan Kotanski <jkotan@mail.desy.de>
#
#    nexdatas is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    nexdatas is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with nexdatas.  If not, see <http://www.gnu.org/licenses/>.
#

""" gitlab tools for converting an svn repoitory to a local git repository"""

import subprocess
import sys
import argparse


class GLFromSVN(object):

    """ GitLab Create a local git repository from an svn repository"""

    def __init__(self, options):
        """ the main program function

        :param options: parser options
        :type options: :class:`argparse.Namespace`
        """
        self.__trunk = options.trunk or "trunk"
        self.__branches = options.branches or "branches"
        self.__tags = options.tags or "tags"
        self.__svnurl = options.svnurl or \
            "https://svn.code.sf.net/p/tango-ds/code/"
        self.__svndir = options.args[0] if len(options.args) > 0 else ""
        self.__url = "%s/%s" % (self.__svnurl, self.__svndir) \
                     if self.__svndir else options.url
        self.__localdir = options.localdir or self.__url.split("/")[-1]
        self.__authors = options.authors or "./authors.txt"
        self.__tagmessage = options.tagmessage or "migrate svn to git"

    def run(self):
        """ the main program function
        """
        # fetch all subgroups
        clonecmd = 'git svn clone %s %s -T %s -b %s -t %s ' \
                   '--authors-file=%s' % (
                       self.__url, self.__localdir, self.__trunk,
                       self.__branches, self.__tags, self.__authors)
        tagcmd = 'cd %s;' \
                 ' \git for-each-ref refs/remotes/origin/tags |' \
                 ' cut -d / -f 5-|' \
                 ' while read ref; ' \
                 ' do git tag -a "$ref" -m"%s" ' \
                 '        "refs/remotes/origin/tags/$ref" && '\
                 '     echo "Add a new tag: $ref"; ' \
                 ' done' % (self.__localdir, self.__tagmessage)
        branchcmd = 'cd %s;' \
                    'git for-each-ref refs/remotes/origin  | ' \
                    'grep -v trunk | grep -v tag |  cut -d / -f 4-| ' \
                    'while read ref; ' \
                    'do git checkout origin/$ref -b $ref &&' \
                    '      echo "Add a new branch:  $ref"; '\
                    'done' % (self.__localdir)
        commands = [clonecmd, tagcmd, branchcmd]
        if self.__url:
            for cmd in commands:
                try:
                    print(cmd)
                    process = subprocess.Popen(
                        cmd,
                        stdout=subprocess.PIPE,
                        shell=True)
                    proc_stdout = process.communicate()[0].strip()
                    if hasattr(proc_stdout, "decode"):
                        print(proc_stdout.decode())
                except Exception as e:
                    print("Error: %s" % (str(e)))


def main():
    """ the main program function
    """

    #: pipe arguments
    pipe = ""
    if not sys.stdin.isatty():
        pp = sys.stdin.readlines()
        #: system pipe
        pipe = "".join(pp)

    description = "Command-line tool create a local git repository" \
                  " from a given svn repository"

    epilog = 'examples:\n' \
        '  glfromsvn DeviceClasses/CounterTimer/DGG2 \n\n' \
        '    - create a local git repository in "DGG2" directory ' \
        'and fetch authors emails from "./authors.txt" \n\n' \

    parser = argparse.ArgumentParser(
        description=description, epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument(
        'args', metavar='svndir', type=str, nargs='?',
        help='svn group directory, '
        'e.g. "DeviceClasses/CounterTimer/DGG2"')
    parser.add_argument(
        "-T", "--trunk",
        help='trunk svn directory, default: trunk',
        dest="trunk", default="trunk")
    parser.add_argument(
        "-b", "--branches",
        help='branch svn directory, default: branches',
        dest="branches", default="branches")
    parser.add_argument(
        "-t", "--tags",
        help='tag svn directory, default: tags',
        dest="tags", default="tags")
    parser.add_argument(
        "-s", "--svn-url",
        help='root svn url, '
        'default: "https://svn.code.sf.net/p/tango-ds/code/"',
        dest="svnurl",
        default="https://svn.code.sf.net/p/tango-ds/code/")
    parser.add_argument(
        "-u", "--url",
        help='full svn url with svn project directory, '
        'e.g.: "https://svn.code.sf.net/p/tango-ds/code/'
        'DeviceClasses/CounterTimer/DGG2"',
        dest="url",
        default="")
    parser.add_argument(
        "-l", "--local-dir",
        help='local git directory, e.g.: "DGG2"',
        dest="localdir",
        default="")
    parser.add_argument(
        "-a", "--authors",
        help='file with authors, '
        'default: "./authors.txt"',
        dest="authors", default="./authors.txt")
    parser.add_argument(
        "-m", "--tag-message",
        help='tag message, '
        'default: "migrate svn to git"',
        dest="tagmessage", default="")

    try:
        options = parser.parse_args()
    except Exception as e:
        sys.stderr.write("Error: %s\n" % str(e))
        sys.stderr.flush()
        parser.print_help()
        print("")
        sys.exit(255)

    #: command-line and pipe arguments
    parg = []
    if hasattr(options, "args"):
        parg = [options.args] if options.args else []
    if pipe:
        parg.append(pipe)
    options.args = parg

    if not options.args and not options.url:
        parser.print_help()
        print("")
        sys.exit(255)

    try:
        command = GLFromSVN(options)
        command.run()
    except Exception as e:
        sys.stderr.write("Error: glfromsvn interrupted with:")
        sys.stderr.write(str(e))
        sys.stderr.flush()
        sys.exit(255)


if __name__ == "__main__":
    main()
